/*!
 * Copyright 2023 <COPYRIGHT HOLDER>
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 * 
 */
import * as pdf from '@trusted-pdf/pdf';

declare enum ImageResolution {
    preview = 72,
    low = 150,
    medium = 200,
    high = 300
}
interface IConvertOptionsImage extends IConvertOptions {
    type: "image";
    /**
     * Default: high (300dpi)
     */
    dpi?: number | ImageResolution;
    pages?: number[];
    /**
     * Максимальное количество промисов, которые могут быть обработаны одновременно.
     * Default: 100
     */
    maxConcurrentPromises?: number;
}
interface IJpegConverterParams {
    canvas: ICanvasParams;
}
interface ICanvas {
    toDataURL(type: string, quality: number): string;
    width: number;
    height: number;
}
interface ICanvasContext {
    canvas?: ICanvas;
    context?: object;
}
interface ICanvasFactory {
    create(width: number, height: number): ICanvasContext;
    destroy(canvasAndContext: ICanvasContext): void;
}
interface ICanvasParams {
    canvasFactory: object;
    url?: ICanvasURLParams;
}
interface ICanvasURLParams {
    /**
     * Path to pdfjs-dist/cmaps.
    */
    cMapUrl: string;
    /**
     * Path to pdfjs-dist/standard_fonts.
    */
    standardFontDataUrl: string;
}
declare class PdfToJpeg {
    private canvasFactory;
    private canvasUrl?;
    constructor(params: IJpegConverterParams);
    /**
     * Converts all pages of pdf to array of jpeg images.
     * @param doc PDF file.
     * @param opt Options.
     */
    convert(doc: Uint8Array | pdf.Document, opt?: IConvertOptionsImage): Promise<Map<number, pdf.IImagePage>>;
    /**
     * Converts page of PDF to image.
     * @param pdfDocument PDF document.
     * @param numPage Page number.
     * @param opt Options.
     */
    private convertPage;
}

interface RuleRegistryParams {
    /**
     * Набор правил копирования объектов.
     */
    copyRules?: pdf.RuleCopy[];
    /**
     * Набор правил удаления объектов.
     */
    deleteRules?: pdf.RuleDelete[];
    /**
     * Набор правил исключения страниц из обычной конвертации.
     */
    exclusionRules?: pdf.RuleExclusion[];
}
/**
 * Представляет собой реестр правил.
 */
declare class RuleRegistry {
    /**
     * Набор правил копирования объектов.
     */
    private copyRules;
    /**
     * Набор правил удаления объектов.
     */
    private deleteRules;
    /**
     * Набор правил исключения страниц из обычной конвертации.
     * Исключенные страницы конвертируются через преобразование в изображения.
     */
    private exclusionRules;
    constructor(params?: RuleRegistryParams);
    /**
     * Добавляет правило копирования объектов.
     * @param rule - Правило копирования объектов.
     */
    addCopyRule(rule: pdf.RuleCopy): void;
    /**
     * Добавляет правило удаления объектов.
     * @param rule - Правило удаления объектов.
     */
    addDeleteRule(rule: pdf.RuleDelete): void;
    /**
     * Добавляет правило исключения страниц из обычной конвертации.
     * @param rule - Правило исключения страниц из обычной конвертации.
     */
    addExclusionRule(rule: pdf.RuleExclusion): void;
    /**
     * Возвращает все правила копирования объектов.
     * @returns Все правила копирования объектов.
     */
    getCopyRules(): pdf.RuleCopy[];
    /**
     * Возвращает правило копирования объектов по названию.
     * @param title - Название правила копирования объектов.
     * @returns Правило копирования объектов или null, если правило не найдено.
     */
    findCopyRule(title: string): pdf.RuleCopy | null;
    /**
     * Возвращает правило копирования объектов по типу.
     * @param type - Тип правила копирования объектов.
     * @returns Правило копирования объектов или null, если правило не найдено.
     */
    findCopyRule<T extends pdf.RuleCopy>(type: pdf.RuleConstructor<T>): T | null;
    /**
     * Возвращает правило копирования объектов по названию.
     * @param title - Название правила копирования объектов.
     * @returns Правило копирования объектов.
     * @throws {RangeError} Если правило не найдено.
     */
    getCopyRule(title: string): pdf.RuleCopy;
    /**
     * Возвращает правило копирования объектов по названию.
     * @param title - Название правила копирования объектов.
     * @param type - Ожидаемый тип правила.
     * @returns Правило копирования объектов.
     * @throws {RangeError} Если правило не найдено.
     * @throws {TypeError} Если правило не соответствует ожидаемому типу.
     */
    getCopyRule<T extends pdf.RuleCopy>(title: string, type: pdf.RuleConstructor<T>): T;
    /**
     * Возвращает правило копирования объектов по типу.
     * @param type - Тип правила копирования объектов.
     * @returns Правило копирования объектов.
     * @throws {RangeError} Если правило не найдено.
     */
    getCopyRule<T extends pdf.RuleCopy>(type: pdf.RuleConstructor<T>): T;
    /**
     * Проверяет правило на существование и соответствие типу.
     * @param rule - Правило для проверки.
     * @param title - Заголовок правила.
     * @param type - Ожидаемый тип правила.
     * @returns Правило.
     * @throws {RangeError} Если правило не найдено.
     * @throws {TypeError} Если правило не соответствует ожидаемому типу.
     */
    private verifyRule;
    private assertNotNullRule;
    /**
     * Возвращает все правила удаления объектов.
     * @returns Все правила удаления объектов.
     */
    getDeleteRules(): pdf.RuleDelete[];
    /**
     * Возвращает правило удаления объектов по названию.
     * @param title - Название правила удаления объектов.
     * @returns Правило удаления объектов или null, если правило не найдено.
     */
    findDeleteRule(title: string): pdf.RuleDelete | null;
    /**
     * Возвращает правило удаления объектов по типу.
     * @param type - Тип правила удаления объектов.
     * @returns Правило удаления объектов или null, если правило не найдено.
     */
    findDeleteRule<T extends pdf.RuleDelete>(type: pdf.RuleConstructor<T>): T | null;
    /**
     * Возвращает правило удаления объектов по названию.
     * @param title - Название правила удаления объектов.
     * @returns Правило удаления объектов.
     * @throws {RangeError} Если правило не найдено.
     */
    getDeleteRule(title: string): pdf.RuleDelete;
    /**
     * Возвращает правило удаления объектов по названию.
     * @param title - Название правила удаления объектов.
     * @param type - Ожидаемый тип правила.
     * @returns Правило удаления объектов.
     * @throws {RangeError} Если правило не найдено.
     * @throws {TypeError} Если правило не соответствует ожидаемому типу.
     */
    getDeleteRule<T extends pdf.RuleDelete>(title: string, type: pdf.RuleConstructor<T>): T;
    /**
     * Возвращает правило удаления объектов по типу.
     * @param type - Тип правила удаления объектов.
     * @returns Правило удаления объектов.
     * @throws {RangeError} Если правило не найдено.
     */
    getDeleteRule<T extends pdf.RuleDelete>(type: pdf.RuleConstructor<T>): T;
    /**
     * Возвращает все правила исключения страниц из обычной конвертации.
     * @returns Все правила исключения страниц из обычной конвертации.
     */
    getExclusionRules(): pdf.RuleExclusion[];
    /**
     * Возвращает правило исключения страниц из обычной конвертации по названию.
     * @param title - Название правила исключения страниц из обычной конвертации.
     * @returns Правило исключения страниц из обычной конвертации или null, если правило не найдено.
     */
    findExclusionRule(title: string): pdf.RuleExclusion | null;
    /**
     * Возвращает правило исключения страниц из обычной конвертации по типу.
     * @param type - Тип правила исключения страниц из обычной конвертации.
     * @returns Правило исключения страниц из обычной конвертации или null, если правило не найдено.
     */
    findExclusionRule<T extends pdf.RuleExclusion>(type: pdf.RuleConstructor<T>): T | null;
    /**
     * Возвращает правило исключения объектов по названию.
     * @param title - Название правила исключения объектов.
     * @returns Правило исключения объектов.
     * @throws {RangeError} Если правило не найдено.
     */
    getExclusionRule(title: string): pdf.RuleExclusion;
    /**
     * Возвращает правило исключения объектов по названию.
     * @param title - Название правила исключения объектов.
     * @param type - Ожидаемый тип правила.
     * @returns Правило исключения объектов.
     * @throws {RangeError} Если правило не найдено.
     * @throws {TypeError} Если правило не соответствует ожидаемому типу.
     */
    getExclusionRule<T extends pdf.RuleExclusion>(title: string, type: pdf.RuleConstructor<T>): T;
    /**
     * Возвращает правило исключения объектов по типу.
     * @param type - Тип правила исключения объектов.
     * @returns Правило исключения объектов.
     * @throws {RangeError} Если правило не найдено.
     */
    getExclusionRule<T extends pdf.RuleExclusion>(type: pdf.RuleConstructor<T>): T;
}

/**
 * Список стандартных имен по спеку.
 */
declare enum FontStNames {
    TimesRoman = "Times-Roman",
    TimesBold = "Times-Bold",
    TimesItalic = "Times-Italic",
    TimesBoldItalic = "Times-BoldItalic",
    Helvetica = "Helvetica",
    HelveticaBold = "Helvetica-Bold",
    HelveticaOblique = "Helvetica-Oblique",
    HelveticaBoldOblique = "Helvetica-BoldOblique",
    Courier = "Courier",
    CourierBold = "Courier-Bold",
    CourierOblique = "Courier-Oblique",
    CourierBoldOblique = "Courier-BoldOblique",
    Symbol = "Symbol",
    ZapfDingbats = "ZapfDingbats"
}
/**
 * Список имен дополнительных стандартных шрифтов не описанных в spec.
 */
declare enum FontExtNames {
    Tahoma = "Tahoma",
    TahomaBold = "Tahoma-Bold",
    ArialNarrow = "ArialNarrow",
    ArialNarrowBold = "ArialNarrow-Bold",
    ArialNarrowBoldItalic = "ArialNarrow-BoldItalic",
    ArialNarrowItalic = "ArialNarrow-Italic",
    ArialBlack = "ArialBlack",
    ArialBlackBold = "ArialBlack-Bold",
    ArialBlackBoldItalic = "ArialBlack-BoldItalic",
    ArialBlackItalic = "ArialBlack-Italic",
    ArialBlack2 = "Arial-Black",
    ArialBlackBold2 = "Arial-Black-Bold",
    ArialBlackBoldItalic2 = "Arial-Black-BoldItalic",
    ArialBlackItalicHyphen = "Arial-Black-Italic",
    Arial = "Arial",
    ArialBold = "Arial-Bold",
    ArialBoldItalic = "Arial-BoldItalic",
    ArialItalic = "Arial-Italic",
    ArialMT = "ArialMT",
    ArialBoldItalicMT = "Arial-BoldItalicMT",
    ArialBoldMT = "Arial-BoldMT",
    ArialItalicMT = "Arial-ItalicMT",
    ArialBoldItalicMTBoldItalic = "Arial-BoldItalicMT-BoldItalic",
    ArialBoldMTBold = "Arial-BoldMT-Bold",
    ArialItalicMTItalic = "Arial-ItalicMT-Italic",
    ArialUnicodeMS = "ArialUnicodeMS",
    ArialUnicodeMSBold = "ArialUnicodeMS-Bold",
    ArialUnicodeMSBoldItalic = "ArialUnicodeMS-BoldItalic",
    ArialUnicodeMSItalic = "ArialUnicodeMS-Italic",
    CourierBoldItalic = "Courier-BoldItalic",
    CourierItalic = "Courier-Italic",
    CourierNew = "CourierNew",
    CourierNewBold = "CourierNew-Bold",
    CourierNewBoldItalic = "CourierNew-BoldItalic",
    CourierNewItalic = "CourierNew-Italic",
    CourierNewPSBoldItalicMT = "CourierNewPS-BoldItalicMT",
    CourierNewPSBoldMT = "CourierNewPS-BoldMT",
    CourierNewPSItalicMT = "CourierNewPS-ItalicMT",
    CourierNewPSMT = "CourierNewPSMT",
    HelveticaBoldItalic = "Helvetica-BoldItalic",
    HelveticaItalic = "Helvetica-Italic",
    SymbolBold = "Symbol-Bold",
    SymbolBoldItalic = "Symbol-BoldItalic",
    SymbolItalic = "Symbol-Italic",
    TimesNewRoman = "TimesNewRoman",
    TimesNewRomanBold = "TimesNewRoman-Bold",
    TimesNewRomanBoldItalic = "TimesNewRoman-BoldItalic",
    TimesNewRomanItalic = "TimesNewRoman-Italic",
    TimesNewRomanPS = "TimesNewRomanPS",
    TimesNewRomanPSBold = "TimesNewRomanPS-Bold",
    TimesNewRomanPSBoldItalic = "TimesNewRomanPS-BoldItalic",
    TimesNewRomanPSBoldItalicMT = "TimesNewRomanPS-BoldItalicMT",
    TimesNewRomanPSBoldMT = "TimesNewRomanPS-BoldMT",
    TimesNewRomanPSItalic = "TimesNewRomanPS-Italic",
    TimesNewRomanPSItalicMT = "TimesNewRomanPS-ItalicMT",
    TimesNewRomanPSMT = "TimesNewRomanPSMT",
    TimesNewRomanPSMTBold = "TimesNewRomanPSMT-Bold",
    TimesNewRomanPSMTBoldItalic = "TimesNewRomanPSMT-BoldItalic",
    TimesNewRomanPSMTItalic = "TimesNewRomanPSMT-Italic"
}
declare const globalFontsMap: Map<string, Uint8Array>;

type UnregisteredFontInfoMap = Map<number, FontInfo[]>;
interface FontInfo {
    name: string;
    resourceName: string;
    dictionary: pdf.PdfDictionary;
}
/**
 * Класс по работе со стандартными шрифтами.
 */
declare class Fonts {
    /**
     * Добавление дополнительного расширенного шрифта.
     * @param name Имя шрифта.
     * @param file Файл шрифта.
     */
    add(name: string, file: Uint8Array): void;
    /**
     * Удаление дополнительного расширенного шрифта.
     * @param name
     */
    delete(name: string): void;
    list(): string[];
    /**
     * Ищет в документе незарегистрированные шрифты.
     * @param file
     * @returns
     */
    static findUnregisteredFonts(file: Uint8Array | pdf.Document): UnregisteredFontInfoMap;
    static findUnregisteredFontsInPage(page: pdf.PdfDictionary): FontInfo[] | null;
    static findUnregisteredFontsInResource(obj: pdf.PdfDictionary): FontInfo[] | null;
    static checkFont(font: pdf.PdfDictionary): boolean;
}

type ConvertOptions = IConvertOptionsImage | IConvertOptionsParse;
interface ConvertResponse {
    file: Uint8Array;
    log: pdf.Logger;
}
interface IConvertOptions {
    typeXRef?: pdf.XRefType;
    type: string;
}
interface IConvertOptionsParse extends IConvertOptions {
    type: "copy";
}
interface IMetadataParams {
    creatorTool?: string;
    producer?: string;
}
interface IPdfaParams {
    canvas: ICanvasParams;
    metadata?: IMetadataParams;
    /**
     * Реестр правил используемых при конвертации. Если параметр не указан,
     * то используется глобальный реестр правил.
     */
    rules?: RuleRegistry;
}
/**
 * Documentation
 */
declare class PDFA {
    private params;
    fonts: Fonts;
    /**
     * Реестр правил используемых при конвертации.
     */
    rules: RuleRegistry;
    constructor(params: IPdfaParams);
    /**
     * Конвертирует файл pdf в формат pdf/a-2b
     * @param file
     * @param params
     * @returns
     */
    convert(file: Uint8Array | pdf.Document, params?: ConvertOptions): Promise<ConvertResponse>;
    /**
     * Добавление в outputIntents цветового профиля RGB.
     * @param doc
     * @param outputIntents
     */
    private createDefaultRGB;
    private getMetadata;
    private getCurrentDateTime;
}

export { ConvertOptions, ConvertResponse, FontExtNames, FontStNames, Fonts, ICanvas, ICanvasContext, ICanvasFactory, ICanvasParams, ICanvasURLParams, IConvertOptions, IConvertOptionsImage, IConvertOptionsParse, IJpegConverterParams, IMetadataParams, IPdfaParams, ImageResolution, PDFA, PdfToJpeg, globalFontsMap };
