/*!
 * Copyright 2023 <COPYRIGHT HOLDER>
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 * 
 */
import * as pdf from '@trusted-pdf/pdf';
import { PdfFlags, IPdfFlagsParams } from '@trusted-pdf/pdf';

interface SignOptions {
    contentLength: number;
    /**
     * CallBack функция, дает возможность изменить Signature Dictionary
     * до сохранения документа и формирования Content на подпись.
     * @returns
     */
    onUpdate?: () => Promise<void>;
    /**
     * CallBack функция подписи Content. Возвращаемое значение записывается в значение Content.
     * @returns
     */
    onSign: (content: Uint8Array) => Promise<Uint8Array>;
}
interface VerifyOptions {
    /**
     * CallBack функция верификации подписи.
     * @param content Проверяемый Content.
     * @param sign Подпись.
     * @returns
     */
    onVerify: (content: Uint8Array, sign: Uint8Array) => Promise<boolean>;
}
interface VerifyResult {
    verifyCallBack: boolean;
}
declare class Signature {
    dict: pdf.PdfDictionary;
    name: string;
    constructor(dict: pdf.PdfDictionary, name: string);
    /**
     * Видимость сигнатуры.
     * Возвращает true если сигнатура видимая и false в противном случае.
     */
    get visibility(): boolean;
    /**
     * Номер страницы на которой находится сигнатура.
     */
    get pageNumber(): number;
    set trustedParams(value: string | null);
    get trustedParams(): string | null;
    /**
     * Подпись сигнатуры.
     * @param options
     */
    sign(options: SignOptions): Promise<void>;
    /**
     * Верификация подписи сигнатуры.
     * @param options
     * @returns
     */
    verify(options: VerifyOptions): Promise<VerifyResult>;
}

interface IRectangle {
    x: number;
    y: number;
    height: number;
    width: number;
}
interface ISignatureParams {
    page: number;
    name: string;
    rectangle?: Partial<IRectangle>;
    stream?: Uint8Array;
    trustedParams?: string;
}
type SignatureList = ReadonlyMap<string, Signature>;
interface ResultVerifyAttestation {
    status: boolean;
    log: string[];
}
declare class Sign {
    #private;
    doc: pdf.Document;
    private static DEFAULT_RECT;
    constructor(doc: pdf.Document);
    /**
     * Проверяет были ли внесены изменения в контент после первой подписи.
     * @returns
     */
    verifyAttestation(): ResultVerifyAttestation;
    /**
     * Ищет все виджеты страницы и заполняет список map.
     * @param map
     * @param page
     * @returns
     */
    private findWidgetsRectOnPage;
    private getRect;
    /**
     * Поиск всех сигнатур в документе.
     * @returns
     */
    private findSignaturesInDocument;
    /**
     * Возвращает список всех сигнатур документа.
     * @returns
     */
    getSignaturesList(): SignatureList;
    /**
     * Поиск сигнатуры по имени или индексу.
     * Индексы начинаются с 0.
     * @param key Имя или индекс сигнатуры.
     */
    findSignature(key: string | number): Signature | null;
    /**
     * Получение сигнатуры по имени или индексу.
     * Индексы начинаются с 0.
     * @param key Имя или индекс сигнатуры.
     */
    getSignature(key: string | number): Signature;
    /**
     * Добавление сигнатуры или массива сигнатур в документ.
     * @param params Параметры добавления.
     * @returns Возвращает список добавленных сигнатур.
     */
    addSignatures(...params: ISignatureParams[]): SignatureList;
    /**
     * Добавление сигнатуры на страницу.
     * @param page
     * @param name
     * @param rectangle
     * @param stream
     * @returns
     */
    private addSignatureOnPage;
}

declare class PdfSigFlags extends PdfFlags<typeof SigFlags> {
    constructor(options?: IPdfFlagsParams<typeof SigFlags>);
}
declare enum SigFlags {
    SignaturesExist = 1,
    AppendOnly = 2
}

export { ISignatureParams, PdfSigFlags, ResultVerifyAttestation, SigFlags, Sign, SignOptions, Signature, SignatureList, VerifyOptions, VerifyResult };
