/*!
 * Copyright 2023 <COPYRIGHT HOLDER>
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 * 
 */
import { Image as Image$1 } from '@pdf-lib/upng';
import * as fontkit from '@pdf-lib/fontkit';

declare class PdfBuffer {
    #private;
    raw: Uint8Array;
    get length(): number;
    constructor(raw?: Uint8Array);
    begin(): this;
    end(endLine?: Uint8Array): this;
    /**
     * Добавляет в текущий массив данных новые данные.
     * @param raw Новые данные.
     * @returns
     */
    append(raw: Uint8Array | PdfBuffer): this;
    /**
     * Добавляет в текущий массив данных новые данные с возможностью добавлять промежуточные данные.
     * @param raw Массив новых данных.
     * @param endLine Разделитель, добавляется только между добавляемыми данными.
     * @returns
     */
    append(raw: Array<Uint8Array | PdfBuffer>, endLine?: Uint8Array): this;
    private addData;
    /**
     * Создает абстрактный подмассив из PdfBuffer.
     * @param begin Начало массива. По умолчанию: 0.
     * @param end Конец массива. По умолчанию: конец массива.
     */
    subarray(begin?: number, end?: number): PdfView;
    /**
     * Преобразует число в Uint8Array в виде строки с добавлением ведущих нулей
     * @param value Число, которое нужно преобразовать
     * @param length Длина результирующего массива(опционально)
     * @returns Массив типа Uint8Array, содержащий преобразованное число в виде строки
     */
    static numberToTextRaw(value: number, length?: number): Uint8Array;
    /**
     * Преобразует строку в массив Uint8Array
     * @param text Строка, которую нужно преобразовать
     * @returns Массив типа Uint8Array, содержащий преобразованную строку
     */
    static stringToRaw(text: string): Uint8Array;
    /**
      * Преобразует число в массив Uint8Array
      * @param num Число, которое нужно преобразовать
      * @param length Длина результирующего массива (опционально)
      * @returns Массив типа Uint8Array, содержащий преобразованное число
     */
    static numberToRaw(num: number, length?: number): Uint8Array;
}

declare class PdfView {
    buffer: PdfBuffer;
    offset: number;
    /**
     * Длинна массива данных.
     */
    readonly length: number;
    constructor(buffer?: PdfBuffer, offset?: number, offsetEnd?: number);
    /**
     * Получение массива данных ссылаясь на массив буфера.
     * @returns
     */
    getRaw(): Uint8Array;
    /**
     * Преобразование массива данных из символьного кода в строку.
     * @returns
     */
    toString(): string;
}

declare class RawData {
    static EMPTY_VIEW: PdfView;
    raw: PdfView;
    isEmptyRaw(): boolean;
}

declare class Header extends RawData {
    version: number;
    comment?: PdfComment;
}

declare class Pages extends PdfDictionary {
    get kids(): PdfArray;
    set kids(value: PdfArray);
    get count(): number;
    set count(value: number);
    onCreate(): void;
}

declare class Catalog extends PdfDictionary {
    get pages(): Pages;
    set pages(value: PdfDictionary);
    onCreate(): void;
}

declare class Trailer extends PdfDictionary {
    get Size(): number;
    set Size(value: number);
    get Prev(): number | null;
    set Prev(value: number | null);
    get Root(): Catalog;
    set Root(value: PdfDictionary);
    get ID(): PdfArray | null;
    set ID(value: PdfArray | null);
    get Info(): PdfDictionary | null;
    set Info(value: PdfDictionary | null);
    get Encrypt(): PdfDictionary | null;
    set Encrypt(value: PdfDictionary | null);
    onCreate(): void;
}

declare enum XRefType {
    XRefTable = 0,
    XRefStream = 1
}
interface XRef extends RawData {
    /**
     * Предыдущий XRef.
     */
    prev?: XRef;
    /**
     * Набор IndirectObject данного XRef.
     */
    objects: Map<number, PdfIndirectObject>;
    /**
     * Документ в котором находится данный XRef.
     */
    doc: Document;
    /**
     * Поле Size указывает на следующий номер IndirectObject для присвоения.
     */
    Size: number;
    trailer: Trailer;
    push(...object: PdfIndirectObject[]): void;
    /**
     * Ищет объект среди своих и среди объектов в предыдущих историях.
     * @param id
     */
    findIndirectObject(id: number): PdfIndirectObject | null;
    /**
     * Ищет и возвращает объект.
     * @param id
     */
    getIndirectObject(id: number): PdfIndirectObject;
}

interface XRefPrevParams {
    xref: XRefTable;
}
interface XRefTrailerParams {
    trailer?: Trailer;
    doc: Document;
}
type XRefTableParams = XRefPrevParams | XRefTrailerParams;
/**
 * ```
 * ... objects ...
 * xref
 * 4 2
 * 0000005208 00000 n
 * 0000005143 00000 n
 * 8 1
 * 0000000807 00000 n
 * trailer
 * <<
 *   /Root ...
 * >>
 * startxref
 * Byte_offset_of_last_cross-reference_section
 * %%EOF
 * ```
 */
declare class XRefTable implements XRef {
    #private;
    prev?: XRef | undefined;
    objects: Map<number, PdfIndirectObject>;
    raw: PdfView;
    get Size(): number;
    set Size(v: number);
    get trailer(): Trailer;
    set trailer(v: Trailer);
    get doc(): Document;
    set doc(v: Document);
    isEmptyRaw(): boolean;
    push(...object: PdfIndirectObject[]): void;
    findIndirectObject(id: number): PdfIndirectObject | null;
    getIndirectObject(id: number): PdfIndirectObject;
    static createByParams(params: XRefTableParams): XRefTable;
}

interface XRefStreamPrevParams {
    xref: XRefStream;
}
type XRefStreamParams = XRefStreamPrevParams | XRefTrailerParams;
/**
 * Представление структуры XRefStream.
 * ```
 * <<
 *  /Type /XRef
 *  /Index [0 32]   %This section has one subsection with 32 objects
 *  /W [1 2 2]      %Each entry has 3 fields: 1, 2 & 2 bytes in width respectively
 *  /Filter /ASCIIHexDecode     %For readability in this example
 *  /Size 32
 *  ...
 * >>
 * stream
 *   00 0000 FFFF
 *   ... cross-references for objects 1 through 10 ...
 *   02 000F 0000
 *   02 000F 0001
 *   02 000F 0002
 *   ... cross-reference for object 14 ...
 *   01 BA5E 0000
 *   ...
 * endstream
 * endobj
 *
 * startxref
 * 54321
 * %%EOF
 * ```
 */
declare class XRefStream extends PdfStream implements XRef {
    #private;
    prev?: XRef | undefined;
    objects: Map<number, PdfIndirectObject>;
    get W(): PdfArray;
    set W(v: PdfArray);
    get Index(): PdfArray;
    set Index(v: PdfArray);
    get Size(): number;
    set Size(v: number);
    get trailer(): Trailer;
    set trailer(v: Trailer);
    get doc(): Document;
    set doc(v: Document);
    push(...object: PdfIndirectObject[]): void;
    findIndirectObject(id: number): PdfIndirectObject | null;
    getIndirectObject(id: number): PdfIndirectObject;
    /**
     * Создает новый XRefStream с использованием параметров.
     * @param params Параметры.
     * @returns
     */
    static createByParams(params: XRefStreamParams): XRefStream;
}

type IResultIs = {
    offset1: number;
    offset2: number;
    ok: boolean;
};
type IResultRead<T extends RawData> = IResultReadTrue<T> | IResultReadFalse;
interface IResultReadTrue<T extends RawData> {
    offset1: number;
    offset2: number;
    ok: true;
    value: T;
}
interface IResultReadFalse {
    ok: false;
}
interface IPairsNumbers {
    num1: number;
    num2: number;
}
declare class Parser {
    #private;
    constructor(doc: Document);
    readXRef(): XRef;
    readObject(index: number): PdfIndirectObjectType;
    readObject<T extends PdfObject>(index: number, type: PdfObjectConstructor<T>, xref?: XRef): T;
    static readComment(buffer: PdfBuffer, offset: number): IResultRead<PdfComment>;
    readHeader(): Header;
    /**
     * Читает любой тип данных из общего raw по индексу.
     * @param index Индекс начала чтении объекта.
     * @returns
     */
    static read(buffer: PdfBuffer, xref: XRef, index: number): IResultRead<PdfIndirectObjectType>;
    static readHeaderObjectStream(buffer: PdfBuffer): Array<IPairsNumbers>;
    readIndirectObject(offset: number, xref?: XRef): PdfObjectType;
    static readDictionary(buffer: PdfBuffer, xref: XRef, index: number): IResultRead<PdfDictionary | PdfStream>;
    static readArray(buffer: PdfBuffer, xref: XRef, index: number): IResultRead<PdfArray>;
    /**
     * Пропускает пробелы и возвращает индекс следующий за пробелом/пробелами.
     * @param buffer
     * @param offset
     * @returns
     */
    static skipWhiteSpaces(buffer: PdfBuffer, offset: number, reverse?: boolean): number;
    /**
     * Пропускает пробелы и комментарии и возвращает индекс следующий за пробелом/пробелами.
     * @param buffer
     * @param offset
     * @returns
     */
    static skipWhiteSpacesAndComments(buffer: PdfBuffer, offset: number): number;
    static readBoolean(buffer: PdfBuffer, index: number): IResultRead<PdfBoolean>;
    static readString(buffer: PdfBuffer, index: number): IResultRead<PdfHexString | PdfLiteralString | PdfName>;
    static readHexString(buffer: PdfBuffer, index: number): IResultRead<PdfHexString>;
    static readLiteralString(buffer: PdfBuffer, index: number): IResultRead<PdfLiteralString>;
    static readNameString(buffer: PdfBuffer, index: number): IResultRead<PdfName>;
    static readNumber(buffer: PdfBuffer, index: number): IResultRead<PdfNumber>;
    static readNull(buffer: PdfBuffer, index: number): IResultRead<PdfNull>;
    private static num;
    /**
     * ```
     * <<
     *  /Type /XRef
     *  /Index [0 32]   %This section has one subsection with 32 objects
     *  /W [1 2 2]      %Each entry has 3 fields: 1, 2 & 2 bytes in width respectively
     *  /Filter /ASCIIHexDecode     %For readability in this example
     *  /Size 32
     *  ...
     * >>
     * stream
     *   00 0000 FFFF
     *   ... cross-references for objects 1 through 10 ...
     *   02 000F 0000
     *   02 000F 0001
     *   02 000F 0002
     *   ... cross-reference for object 14 ...
     *   01 BA5E 0000
     *   ...
     * endstream
     * endobj
     *
     * startxref
     * 54321
     * %%EOF
     * ```
     */
    private readXRefStream;
    private getLastEOF;
    private readXRefTable;
    private static isLiteralString;
    /**
     * Проверяет не вышел ли индекс за предел файла.
     * @param offset2 Проверяемый индекс.
     */
    private static isEndFile;
    /**
     * Проверяет является ли данный индекс началом stream.
     * @param index
     * @param length
     * @returns Если true то возвращает индексы содержимого stream.
     */
    private static isStream;
    /**
     *  Проверяет указывает ли данный индекс на начало объекта.
     * @param index Индекс начала объекта.
     * @returns Если результат положительный то ответ содержит номер объекта.
     */
    private static isObject;
    /**
     * Проверяет указывает ли данный индекс на начало ссылки объекта.
     * @param index Индекс начала ссылки.
     * @returns Если результат положительный то ответ содержит номер ссылки.
     */
    private static isRef;
    /**
     * Проверяет указывает ли данный индекс на начало записи типа 0123 0123 EndWord.
     * @param index Индекс начала записи.
     * @returns Если результат положительный то ответ содержит номер ссылки.
     */
    private static isPairNumbers;
    /**
     * Проверяет первый символ или слово и ищет замыкающий символ или слово.
     * @param index Индекс первого символа.
     * @param start Открывающий символ или слово.
     * @param end Закрывающий символ или слово.
     * @returns Если true то возвращает индексы значения между start и end.
     */
    private static isBetween;
    /**
     * Проверяет является ли текущий символ проверяемого типа.
     * Если этот символ относится к данному типу то метод перебирает последующие символы,
     * до тех пор пока не закончится данный тип значения.
     * @param index Текущий индекс символа
     * @param reverse Если true то перебор индексов идет в обратном направлении. По умолчанию false.
     * @returns Если результат положительный то в offset2 лежит индекс последнего символа значения.
     */
    private static isType;
    /**
     * Проверяет входит ли текущий символ в целевой массив.
     * @param index Индекс текущего символа.
     * @param array Массив со значениями которого сравнивается текущий символ.
     * @returns
     */
    private static isChar;
    /**
     * Проверяет является ли следующий набор символов, начиная с индекса, проверяемым словом.
     * @param index Индекс начала слова.
     * @param word Проверяемое слово.
     * @param reverse Если true то перебор индексов идет в обратном направлении. По умолчанию false.
     * @returns Если результат положительный то в offset2 лежит индекс последнего символа слова.
     */
    static isWord(buffer: PdfBuffer, index: number, word: Uint8Array, reverse?: boolean): IResultIs;
    /**
     * Перебирает символы до тех пор пока не встретит искомое слово.
     * @param index Индекс начала поиска.
     * @param word Искомое слово.
     * @returns Если результат положительный то в offset1 и offset2 лежат индексы начала и конца слова.
     */
    private static findWord;
    private static findType;
    private static getNumberByOffset;
    private getByte;
    /**
     * Проверяет, является ли данный символ допустимой буквой оператора.
     * Буквы оператора включают A-Z, a-z, *, ', и ".
     *
     * @param char - Проверяемый символ.
     * @returns `true`, если символ является допустимой буквой оператора, `false` в противном случае.
     */
    private static isOperatorLetter;
    private static readOperatorTag;
    private static readOperatorOrObject;
    static readOperator(buffer: PdfBuffer, xref: XRef, index: number): IResultRead<PdfOperator>;
}

interface IObjectStreamParams extends IPdfDictionaryParams {
    objects?: PdfDictionary[];
}
/**
 * Представление для структуры ObjectStream.
 */
declare class ObjectStream extends PdfStream {
    private n;
    private first;
    private extends;
    /**
     * @internal
     */
    pairsNumbers?: Array<IPairsNumbers>;
    /**
     * @internal
     */
    objects: Map<number, PdfObjectType>;
    constructor(param?: IObjectStreamParams);
    /**
     *  Количество объектов хранящихся в данном потоке.
     */
    get N(): number;
    /**
     *  Количество объектов хранящихся в данном потоке.
     */
    set N(v: number);
    /**
     * Байтовый отступ в распакованном потоке указывающий на первый сжатый объект.
     */
    get First(): number;
    /**
     * Байтовый отступ в распакованном потоке указывающий на первый сжатый объект.
     */
    set First(v: number);
    /**
     *  Ссылка на другой объектный поток.
     */
    get Extends(): ObjectStream | null;
    /**
     * Возвращает объект по индексу из внутреннего стрима.
     * @param index индекс объекта.
     * @returns
     */
    getObject(index: number): PdfObjectType;
    onCreate(): void;
}

type PdfIndirectObjectType = PdfIndirectObject | PdfObjectType;
declare enum IndirectObjectType {
    inUse = "n",
    free = "f",
    compressed = "c"
}
interface IPdfIndirectObjectParams extends IPdfObjectParams {
    xref?: XRef;
    id: number;
    generation?: number;
    offset?: number;
    index?: number;
    objStream?: number;
    type?: IndirectObjectType;
}
declare class PdfIndirectObject extends PdfObject {
    id: number;
    generation: number;
    offset: number;
    index: number;
    objStream: number;
    type: IndirectObjectType;
    /**
     * Хранит в себе внутренние значение объекта. Объект считывается из массива данных только при необходимости.
     * @internal
    */
    privateValue?: PdfObjectType;
    constructor(param: IPdfIndirectObjectParams);
    getValue(xref?: XRef): PdfObjectType;
    get value(): PdfObjectType;
    set value(v: PdfObjectType);
    getHistory(): boolean;
    protected onChange(): void;
}

interface IPdfArrayParams extends IPdfObjectParams {
    value?: PdfIndirectObjectType[];
}
declare class PdfArray extends PdfObject implements Iterable<PdfIndirectObjectType> {
    /**
     * @internal
     */
    value: PdfIndirectObjectType[];
    constructor(param?: IPdfArrayParams);
    get id(): number;
    [Symbol.iterator](): Iterator<PdfIndirectObjectType, unknown, undefined>;
    get length(): number;
    find(index: number): PdfObject | null;
    find<T extends PdfObject>(index: number, type: PdfObjectConstructor<T>): T | null;
    get(index: number): PdfObject;
    get<T extends PdfObject>(index: number, type: PdfObjectConstructor<T>): T;
    push(...obj: PdfIndirectObjectType[]): void;
}

interface IPdfBooleanParams extends IPdfObjectParams {
    value?: boolean;
}
declare class PdfBoolean extends PdfObject {
    #private;
    constructor(param?: IPdfBooleanParams);
    get value(): boolean;
    set value(v: boolean);
}

interface IPdfStringParams extends IPdfObjectParams {
    value?: string;
}
declare class PdfString extends PdfObject {
    #private;
    constructor(param?: IPdfStringParams);
    get value(): string;
    set value(v: string);
    protected onSetValue(v: PdfString): void;
    toBinary(): Uint8Array;
}

declare class PdfName extends PdfObject {
    #private;
    constructor(param?: IPdfStringParams);
    get value(): string;
    set value(v: string);
}

declare class PdfNull extends PdfObject {
    constructor(param?: IPdfObjectParams);
    get value(): null;
}

interface IPdfNumberParams extends IPdfObjectParams {
    value?: number;
}
declare class PdfNumber extends PdfObject {
    #private;
    constructor(param?: IPdfNumberParams);
    get value(): number;
    set value(v: number);
    protected onConvert(obj: PdfObject): void;
    protected onSetValue(v: PdfNumber): void;
    reWriteValue(v: number): void;
}

/**
 * Параметры создания класса FilterList.
 */
interface IFilterListParams {
    pdfStream: PdfStream;
}
/**
 * Управляет фильтрами внутри PdfStream.
 */
declare class FilterList {
    /**
     * PdfStream в котором идет управление фильтрами.
     */
    private pdfStream;
    /**
     * Массив текущих фильтров.
     */
    private filters;
    constructor(params: IFilterListParams);
    /**
     * Кодирует данные используя текущие установленные фильтры.
     * @param value входящие данные.
     * @returns
     */
    encode(value: Uint8Array): Uint8Array;
    /**
     * Декодирует данные используя текущие установленные фильтры.
     * @param value входящие данные.
     * @returns
     */
    decode(value: Uint8Array): Uint8Array;
    /**
     * Возвращает массив Filter из PdfStream.
     * @returns
     *
     * @remarks
     * Функция формирует новый список фильтров на основе найденных имен и параметров,
     * а не из кэша (this.filters).
     */
    list(): ReadonlyArray<Filter>;
    /**
     * Возвращает массив Filter из PdfStream.
     * @returns
     */
    private getPdfStreamFilters;
    /**
     * Задает новый список фильтров, удаляя предыдущий.
     * @param filters
     * @returns
     */
    setNewFilters(...filters: Filter[]): void;
    /**
     * Возвращает массив Filter из PdfStream.
     * @returns
     */
    private findFilters;
    /**
     * Возвращает массив DecodeParms из PdfStream.
     * @returns
     */
    private findDecodeParms;
    copy(target: PdfStream): FilterList;
}

declare class PdfStream extends PdfDictionary {
    #private;
    filters: FilterList;
    stream: PdfView;
    constructor(param?: IPdfObjectParams);
    get Length(): number;
    set Length(value: number);
    set rawValue(raw: Uint8Array);
    /**
     * Значение с фильтрами что находится в raw. Закодированное.
     */
    get rawValue(): Uint8Array;
    /**
     * Значение без фильтров. Декодированное.
     */
    get value(): Uint8Array;
    /**
     * Value не должно быть закодировано фильтрами.
     */
    set value(v: Uint8Array);
    protected onCreate(): void;
    protected onConvert(obj: PdfObject): void;
}

type PdfObjectType = PdfBoolean | PdfNumber | PdfString | PdfName | PdfArray | PdfDictionary | PdfStream | PdfNull;

interface IPdfDictionaryParams extends IPdfObjectParams {
    values?: Map<string, PdfIndirectObjectType>;
}
declare class PdfDictionary extends PdfObject {
    /**
     * @internal
     */
    values: Map<string, PdfIndirectObjectType>;
    constructor(param?: IPdfDictionaryParams);
    /**
     * Возвращает id если есть PdfIndirectObject родитель.
     * Иначе вернет ошибку.
     */
    get id(): number;
    get items(): ReadonlyMap<string, PdfIndirectObjectType>;
    protected onConvert(obj: PdfObject): void;
    /**
     * Проверяет наличие элемента с заданным ключом.
     * @param key - Ключ для поиска.
     * @returns `true`, если элемент с заданным ключом существует, иначе `false`.
     */
    has(key: string | PdfName): boolean;
    find(key: string | PdfName): PdfObjectType | null;
    find<T extends PdfObject>(key: string | PdfName, type: PdfObjectConstructor<T>): T | null;
    get(key: string | PdfName): PdfObjectType;
    get<T extends PdfObject>(key: string | PdfName, type: PdfObjectConstructor<T>): T;
    set(key: string | PdfName, obj: PdfIndirectObjectType): void;
    findOrSet<T extends PdfObject>(key: string | PdfName, type: PdfObjectConstructor<T>, ref?: boolean): T;
    /**
     * Удаляет элемент с заданным ключом.
     * @param key - Ключ для поиска.
     * @returns `true`, если элемент с заданным ключом был удален, иначе `false`.
     */
    remove(key: string | PdfName): boolean;
    /**
     * Ищет значение с заданным ключом в текущем объекте или в его родительских объектах,
     * поднимаясь по иерархии объектов, пока не будет найдено значение с заданным ключом или пока не достигнется верхний уровень.
     *
     * @param key - Ключ для поиска.
     * @returns Найденное значение или `null`, если значение не найдено.
     */
    findInherited(key: string | PdfName): PdfObjectType | null;
    /**
     * Ищет значение с заданным ключом и определенного типа в текущем объекте или в его родительских объектах,
     * поднимаясь по иерархии объектов, пока не будет найдено значение с заданным ключом и типом, или пока не достигнется верхний уровень.
     *
     * @param key - Ключ для поиска.
     * @param type - Ожидаемый тип значения.
     * @returns Найденное значение заданного типа или `null`, если значение не найдено.
     */
    findInherited<T extends PdfObject>(key: string | PdfName, type: PdfObjectConstructor<T>): T | null;
    /**
     * Получает значение с заданным ключом в текущем объекте или в его родительских объектах,
     * поднимаясь по иерархии объектов, пока не будет найдено значение с заданным ключом.
     *
     * @param key - Ключ для поиска.
     * @returns Найденное значение.
     * @throws Error, если значение не найдено.
     */
    getInherited(key: string | PdfName): PdfObjectType;
    /**
     * Получает значение с заданным ключом и определенного типа в текущем объекте или в его родительских объектах,
     * поднимаясь по иерархии объектов, пока не будет найдено значение с заданным ключом и типом.
     *
     * @param key - Ключ для поиска.
     * @param type - Ожидаемый тип значения.
     * @returns Найденное значение заданного типа.
     * @throws Error, если значение не найдено или не соответствует ожидаемому типу.
     */
    getInherited<T extends PdfObject>(key: string | PdfName, type: PdfObjectConstructor<T>): T;
}

interface Filter {
    name: string;
    params?: PdfDictionary;
}
declare abstract class FilterCodec {
    private static items;
    /**
     * Применяет кодировку фильтров к data.
     * @param data Данные, которые необходимо кодировать.
     * @param filters Один или несколько фильтров, которыми нужно кодировать data.
     * Если передано несколько фильтров, то они будут применены в обратном порядке.
     * @returns Возвращает кодированные данные.
     */
    static encode(data: Uint8Array, ...filters: Filter[]): Uint8Array;
    /**
     * Применяет раскодировку фильтров к data.
     * @param data Данные, которые необходимо раскодировать.
     * @param filters Один или несколько фильтров, которыми нужно раскодировать data.
     * Если передано несколько фильтров, то они будут применены в прямом порядке.
     * @returns Возвращает раскодированные данные.
     */
    static decode(data: Uint8Array, ...filters: Filter[]): Uint8Array;
}

type FilterName = Names.ASCIIHexDecode | Names.ASCII85Decode | Names.LZWDecode | Names.FlateDecode | Names.RunLengthDecode | Names.DCTDecode;
interface IFilter {
    name: FilterName;
    /**
     * Кодирует входящие данные по параметрам.
     * @param data Входящие данные.
     * @param params Параметры кодирования.
     */
    encode(data: Uint8Array, params: PdfDictionary | null): Uint8Array;
    /**
     * Декодирует входящие данные по параметрам.
     * @param data Входные данные.
     * @param params Параметры декодирования.
     */
    decode(data: Uint8Array, params: PdfDictionary | null): Uint8Array;
}

declare class FlateDecode implements IFilter {
    readonly name: FilterName;
    encode(data: Uint8Array, params?: PdfDictionary | null): Uint8Array;
    decode(data: Uint8Array, params?: PdfDictionary | null): Uint8Array;
}

declare class ASCII85Decode implements IFilter {
    readonly name: FilterName;
    encode(data: Uint8Array): Uint8Array;
    decode(data: Uint8Array): Uint8Array;
}

declare abstract class Predictor {
    static decode(data: Uint8Array, params: PdfDictionary): Uint8Array;
    private static decodeTiff;
    private static decodePNG;
    static encode(data: Uint8Array, params: PdfDictionary): Uint8Array;
    private static encodeTiff;
}

declare class ASCIIHexDecode implements IFilter {
    readonly name: FilterName;
    encode(data: Uint8Array): Uint8Array;
    decode(data: Uint8Array): Uint8Array;
}

declare abstract class StringConverter {
    /**
     * Раскодирует hex строку.
     * @param raw Данные, содержащие закодированную строку.
     * @returns Раскодированная строка символов.
     */
    static readHex(raw: Uint8Array): string;
    /**
     * Кодирует входной текст в шестнадцатеричную строку или в байтовый массив в формате UTF-8 или UTF-16BE.
     * @param {string} text - Входной текст, который требуется закодировать.
     * @param {boolean} string - Опциональный параметр. Если установлен в `true`, функция вернет шестнадцатеричную строку. По умолчанию возвращается байтовый массив.
     * @returns {Uint8Array | string} Закодированный текст в формате UTF-8 или UTF-16BE в виде байтового массива или шестнадцатеричной строки.
     */
    static writeHex(text: string, string: true): string;
    static writeHex(text: string): Uint8Array;
    /**
     * Раскодирует literal строку.
     * @param raw Данные, содержащие закодированную строку.
     * @returns Раскодированная строка символов.
     */
    static readLiteral(raw: Uint8Array): string;
    /**
     * Кодирует строку символов в literal строку.
     * @param raw Строка, которую нужно закодировать.
     * @returns Literal строка.
     */
    static writeLiteral(text: string): Uint8Array;
    /**
     * Раскодирует name строку.
     * @param raw Данные, содержащие закодированную строку.
     * @returns Раскодированная строка символов.
     */
    static readName(raw: Uint8Array): string;
    /**
     * Кодирует строку символов в Name строку.
     * @param raw Строка, которую нужно закодировать.
     * @returns Name строка.
     */
    static writeName(text: string): Uint8Array;
}

declare enum Encoding {
    Utf8 = "utf-8",
    Utf16Be = "utf-16be",
    Binary = "binary"
}
/**
 * Генерация уникального hex ID.
 * @param length Размер ID.
 * @returns
 */
declare function generateUniqueID(length?: number): Uint8Array;
/**
 * Генерация UUID v4.
 * @returns
 */
declare function generateUUID(): string;
/**
 * Преобразует Uint8Array в строку Base64 без использования atob и btoa.
 * @param {Uint8Array} uint8Array - Исходный Uint8Array.
 * @returns {string} Строка Base64.
 */
declare function toBase64(uint8Array: Uint8Array): string;
/**
 * Преобразует строку Base64 в Uint8Array без использования atob и btoa.
 * @param {string} base64String - Строка Base64.
 * @returns {Uint8Array} Результирующий Uint8Array.
 */
declare function fromBase64(base64String: string): Uint8Array;
/**
 * Проверяет, содержит ли строка маркер последовательности байтов UTF-16BE.
 * @param str Строка, которую необходимо проверить.
 * @returns Результат проверки.
 */
declare function hasUtf16BeBOM(str: Uint8Array): boolean;
/**
 * Проверяет, содержит ли строка маркер последовательности байтов UTF-8.
 * @param str Строка, которую необходимо проверить.
 * @returns Результат проверки.
 */
declare function hasUtf8BOM(str: Uint8Array): boolean;
/**
 * Переводит строку hex значений в binary данные.
 * @param hexString Hex строка, которую необходимо преобразовать.
 * @returns Полученные binary данные.
 */
declare function hexToBinary(hexString: string): Uint8Array;
/**
 * Кодирует строку в UTF-16BE.
 * @param str Строка, которую необходимо закодировать.
 * @returns Закодированная строка.
 */
declare function toUtf16Be(str: string): Uint8Array;
/**
 * Приводит hex значение в строке в 2-байтный формат.
 * @param hex Hex строка, которую необходимо преобразовать.
 * @returns Преобразованная строка.
 */
declare function hexTo2ByteHex(hex: string): string;
/**
 * Приводит hex значение в строке в 1-байтный формат.
 * @param hex Hex строка, которую необходимо преобразовать.
 * @returns Преобразованная строка.
 */
declare function ensure1ByteHex(hex: string): string;
/**
 * Форматирует строку hex значений соответственно указанному количеству байт (1 или 2).
 * @param hex Строка hex значений.
 * @param as2Byte Сколько байт должно занимать hex значение. true - 2 байта, false - 1 байт.
 * @returns Преобразованная строка.
 */
declare function formatHex(hex: string, as2Byte: boolean): string;
/**
 * Проверяет, есть ли в строке символы, занимающие больше 1 байта.
 * @param text Строка, которую необходимо проверить.
 * @returns Результат проверки.
 */
declare function hasMoreThan1ByteChars(text: string): boolean;
/**
 * Раскодирует binary кодировку.
 * @param str Строка, которую необходимо раскодировать.
 * @returns Раскодированная строка символов.
 */
declare function fromBinary(str: Uint8Array): string;
/**
 * Кодирует binary кодировкой.
 * @param str Строка, которую необходимо кодировать.
 * @returns Закодированная строка
 */
declare function toBinary(str: string): Uint8Array;
/**
 * Обрабатывает экранированные специальные символы.
 * @param raw Строка, которую необходимо обработать.
 * @returns Обработанная строка.
 */
declare function unescapeSpecialSymbols(raw: Uint8Array): Uint8Array;
/**
 * Экранирует специальные символы.
 * @param str Строка, которую необходимо экранировать.
 * @returns Экранированная строка.
 */
declare function escapeSpecialSymbols(text: string): string;
/**
 * Раскодирует строку с указанной кодировкой.
 * @param str Закодированная строка.
 * @param encoding Кодировка строки.
 * @returns Раскодированная строка символов.
 */
declare function decodeString(str: Uint8Array, encoding: Encoding): string;
declare function isEqualRaw(array1: Uint8Array, array2: Uint8Array): boolean;

declare class PdfHexString extends PdfString {
    reWriteValue(v: string | Uint8Array): void;
}

declare class PdfLiteralString extends PdfString {
    reWriteValue(v: string | Uint8Array): void;
}

declare class Writer {
    #private;
    constructor(doc: Document);
    /**
     * Проверяет что все xref в документе одного типа.
     * @param typeXRef
     * @param xref
     * @returns
     */
    private isCorrectXRefType;
    /**
     * Сериализация документа.
     * @returns
     */
    save(): Uint8Array;
    /**
     * Добавление Header.
     * @param buf
     */
    private wHeader;
    private wXRef;
    private getAllObjects;
    /**
     * Удаляет из списка PdfIndirectObject у которых тип Free (удален).
     * @param objects
     * @returns
     */
    private removeFreeObjects;
    /**
     * Получение объектов XRefTable.
     * @param xref
     * @returns
     */
    private getXRefTableObjects;
    /**
     * Получение объектов XRefStream.
     * @param xref
     * @returns
     */
    private getXRefStreamObjects;
    /**
     * Добавляет в objStream объекты из mapObjects а также формирует и добавляет raw в stream.
     * @param objStream
     * @param mapObjects
     */
    private wObjectStream;
    /**
     * Формирование raw stream для XRefStream.
     * @param xref
     * @returns
     */
    private wXRefStreamValue;
    private wXRefStream;
    private wXRefTable;
    /**
     * Создает линию таблицы обновления без eol в конце строки.
     * @param obj
     * @param flag
     * @returns
     */
    private createLineTable;
    private writeObject;
    private wIndObj;
    write(buf: PdfBuffer, obj: PdfIndirectObjectType, writeAsNotRef?: boolean, writeToBuffer?: boolean): Uint8Array;
    private wDictionary;
    private wStream;
    private wArray;
    private wRefObj;
    private wBoolean;
    private wNumber;
    private wName;
    private wString;
    private static addPadding;
    static numberToUint8(obj: PdfNumber): Uint8Array;
    static hexToUint8(obj: PdfHexString): Uint8Array;
    static literalToUint8(obj: PdfLiteralString): Uint8Array;
}

declare enum RegistryTypes {
    PdfView = 0,
    ObjectStream = 1,
    Writer = 2,
    PdfIndirectObject = 3,
    Parser = 4,
    Copier = 5,
    PdfDictionary = 6,
    PdfArray = 7
}
/**
 * Класс предназначен для обхода проблемы с циркулярными зависимостями.
 * Для регистрации класса необходимо добавить метод get, расширить enum RegistryTypes
 * и зарегистрировать класс в файле register.
 */
declare abstract class Registry {
    static items: Record<number, unknown>;
    static get(key: RegistryTypes.PdfIndirectObject): typeof PdfIndirectObject;
    static get(key: RegistryTypes.Parser): typeof Parser;
    static get(key: RegistryTypes.Writer): typeof Writer;
    static get(key: RegistryTypes.Copier): typeof Copier;
    static get(key: RegistryTypes.PdfView): typeof PdfView;
    static get(key: RegistryTypes.ObjectStream): typeof ObjectStream;
    static get(key: RegistryTypes.PdfDictionary): typeof PdfDictionary;
    static get(key: RegistryTypes.PdfArray): typeof PdfArray;
    static set(key: RegistryTypes, value: unknown): void;
}

/**
 * Данные JPEG изображения.
 */
interface IImageJpegInfo extends IBaseImage {
    /**
     * Тип изображения: JPEG.
     */
    type: ImageTypes.JPEG;
    /**
     * Дополнительная информация о JPEG.
     */
    info: IJpegInfo;
}
/**
 * Дополнительная информация о JPEG.
 */
interface IJpegInfo extends IBaseInfo {
}
declare abstract class JpegInfo {
    static getInfo(jpeg: Uint8Array): IImageJpegInfo;
}

declare module "@pdf-lib/upng" {
    export default class UPNG {
        static decode(data: Uint8Array): Image$1;
        static toRGBA8(upng: Image$1): Uint8Array[];
    }
}
/**
 * Данные PNG изображения.
 */
interface IImagePngInfo extends IBaseImage {
    /**
     * Тип изображения: PNG.
     */
    type: ImageTypes.PNG;
    /**
     * Дополнительная информация о PNG.
     */
    info: IPngInfo;
}
/**
 * Дополнительная информация о PNG.
 */
interface IPngInfo extends IBaseInfo {
    /**
     * Альфа-канал для PNG изображения.
     */
    alphaChannel?: Uint8Array;
}
declare abstract class PngInfo {
    static getInfo(image: Uint8Array): IImagePngInfo;
    private static splitAlphaChannel;
}

declare enum ColorNames {
    DeviceGray = 1,
    DeviceRGB = 3,
    DeviceCMYK = 4
}
declare enum ImageTypes {
    PNG = 0,
    JPEG = 1
}
type ColorSpace = keyof typeof ColorNames;
type ImageInfo = IImagePngInfo | IImageJpegInfo;
/**
 * Базовый интерфейс изображения.
 */
interface IBaseImage {
    /**
     *  Буфер изображения.
     */
    buf: Uint8Array;
    /**
     * Информация об изображении.
     */
    info: IBaseInfo;
}
/**
 * Базовая информация об изображении.
 */
interface IBaseInfo {
    /**
     *  Количество бит на компонент.
     */
    bitsPerComponent: number;
    /**
     * Ширина изображения.
     */
    width: number;
    /**
     * Высота изображения.
     */
    height: number;
    /**
     * Цветовое пространство изображения.
     */
    colorSpace: ColorSpace;
}
declare abstract class ImageAnalyzer {
    /**
     * Получает изображение на основе буфера изображения.
     * @param buf Буфер изображения.
     * @returns Изображение в формате ImagePng или ImageJpeg.
     */
    static getImageInfo(buf: Uint8Array): ImageInfo;
}

interface IPdfFlagsParams<T> extends IPdfNumberParams {
    flags?: Array<keyof T>;
}
declare class PdfFlags<T> extends PdfNumber {
    protected type: T;
    constructor(type: T, options?: IPdfFlagsParams<T>);
    /**
     * Проверяет включен ли флаг.
     * @param v
     * @returns
     */
    has(v: keyof T): boolean;
    /**
     * Включает указанный флаг.
     * @param v
     */
    set(...v: Array<keyof T>): void;
    /**
     * Возвращает массив всех включенных флагов.
     * @returns
     */
    get(): Array<keyof T>;
    /**
     * Выключает указанный флаг.
     * @param v
     */
    unset(...v: Array<keyof T>): void;
}

declare class PdfFontFlags extends PdfFlags<typeof FontFlags> {
    constructor(options?: IPdfFlagsParams<typeof FontFlags>);
}
declare enum FontFlags {
    /**
     * All glyphs have the same width (as opposed to proportional or variable-pitch fonts, which have different widths).
     */
    FixedPitch = 1,
    /**
     * Glyphs have serifs, which are short strokes drawn at an angle on the topand bottom of glyph stems. (Sans serif fonts do not have serifs.)
     */
    Serif = 2,
    /**
     * Font contains glyphs outside the Standard Latin character set. This flagand the Nonsymbolic flag shall not both be set or both be clear.
     */
    Symbolic = 4,
    /**
     * Glyphs resemble cursive handwriting.
     */
    Script = 8,
    /**
     * Font uses the Standard Latin character set or a subset of it. This flag andthe Symbolic flag shall not both be set or both be clear.
     */
    Nonsymbolic = 32,
    /**
     * Glyphs have dominant vertical strokes that are slanted.
     */
    Italic = 64,
    /**
     * Font contains no lowercase letters; typically used for display purposes, such as for titles or headlines.
     */
    AllCap = 65536,
    /**
     * Font contains both uppercase and lowercase letters. The uppercaseletters are similar to those in the regular version of the same typefacefamily. The glyphs for the lowercase letters have the same shapes as thecorresponding uppercase letters, but they are sized and their proportionsadjusted so that they have the same size and stroke weight as lowercaseglyphs in the same typeface family.
     */
    SmallCap = 131072,
    /**
     * See description after Note 1 in this subclause.
     */
    ForceBold = 262144
}

interface ICustomFontParams {
    /**
     * Массив байтов, содержащий данные шрифта.
     */
    fontData: Uint8Array;
}
/**
 * Класс `CustomFont` представляет пользовательский шрифт в формате PDF.
 * Этот класс позволяет работать с пользовательскими шрифтами, включая доступ к различным характеристикам и свойствам шрифта.
 */
declare class CustomFont {
    /**
     * Массив байтов, содержащий данные шрифта.
     */
    readonly fontData: Uint8Array;
    /**
     * Объект шрифта, созданный с использованием библиотеки fontkit.
     */
    readonly font: fontkit.Font;
    /**
     * Создает новый экземпляр класса `PdfCustomFont`.
     * @param {ICustomFontParams} params - Параметры шрифта, включая данные шрифта в виде Uint8Array.
     */
    constructor(params: ICustomFontParams);
    /**
     * Получает коэффициент масштабирования для шрифта.
     */
    get scale(): number;
    /**
     * Получает ограничивающий прямоугольник (BBox) шрифта.
     */
    get fontBBox(): number[];
    /**
     * Получает значение ascent (восхождение) шрифта.
     */
    get ascent(): number;
    /**
     * Получает значение descent (спуск) шрифта.
     */
    get descent(): number;
    /**
     * Получает значение capHeight (высота заглавных букв) шрифта.
     */
    get capHeight(): number;
    /**
     * Получает значение xHeight (высота строчных букв) шрифта.
     */
    get xHeight(): number;
    /**
     * Получает угол наклона шрифта (italic angle).
     */
    get italicAngle(): number;
    /**
     * Получает флаги шрифта.
     */
    get flags(): PdfFontFlags;
    /**
     * Получает подтип шрифта.
     */
    get subtype(): string;
    /**
     * Получает отображение CID в GID (Character ID в Glyph ID).
     */
    get cidToGIDMap(): string;
    /**
     * Получает массив ширин глифов шрифта.
     */
    get widths(): number[];
    /**
     * Получает массив глифов шрифта.
     */
    glyphs(): fontkit.Glyph[];
    /**
     * Удаляет дубликаты и сортирует массив глифов.
     */
    private sortedUniq;
    /**
     * Получает идентификатор глифа.
     * @param {fontkit.Glyph} glyph - Глиф шрифта.
     * @returns {number} Идентификатор глифа.
     */
    private glyphId;
    /**
     * Возвращает массив поддерживаемых языков.
     */
    supportedLanguages(): string[];
}

/**
 * Класс Fonts предоставляет методы для работы с шрифтами в PDF.
 */
declare class Fonts {
    /**
     * Создает экземпляр пользовательского шрифта на основе данных шрифта в формате Uint8Array.
     * @param {Uint8Array} fontData - Данные шрифта в формате Uint8Array.
     * @returns {CustomFont} Экземпляр пользовательского шрифта.
     */
    static getCustom(fontData: Uint8Array): CustomFont;
}

/**
 * Ключ - ID правила.
 * Значение - массив изменений по правилу в документе.
 */
type LogRules = {
    [key: string]: Array<Record$1>;
};
/**
 * Запись изменения.
 */
type Record$1 = {
    [key: string]: string | number | boolean | Array<string | number>;
};
/**
 * Класс логирования изменений в документе.
 */
declare class Logger {
    /**
     * Записи изменений.
     */
    records: LogRules;
    /**
     * Возвращает количество записей в логе.
     * @returns
     */
    size(): number;
    /**
     * Очистка лога.
     */
    clear(): void;
    /**
   * Добавление записи.
   * @param key - Ключ для записи.
   * @param value - Значение записи.
   */
    addValue(key: string, value: Record$1): void;
}

interface IDocumentCreateStreamParams {
    dict?: Record<string, PdfIndirectObjectType>;
    stream?: Uint8Array;
    filters?: Filter[];
}
interface CopyResponse {
    doc: Document;
    log: Logger;
}
interface CopyParams {
    /**
     * Массив правил копирования.
     */
    rules?: Rules;
    /**
     * Тип структуры XRef.
     */
    typeXRef?: XRefType;
}
interface Font {
    unicode: Map<string, string>;
    dictionary: PdfDictionary;
}
/**
 * Карта страниц с ключом по номеру странниц.
 * Номер страниц начинается с 1.
 */
type PagesList = ReadonlyMap<number, PdfDictionary>;
declare class Document {
    #private;
    fonts: Map<string, Font>;
    header: Header;
    buffer: PdfBuffer;
    get xref(): XRef;
    set xref(v: XRef);
    /**
     * Чтение и распознание PDF документа.
     * @param file PDF документ.
     */
    read(file: Uint8Array): void;
    /**
     * Сохранение текущих изменений, добавляя новый блок EOF.
     * Если изменений нет то сохранение не выполняется.
     * @returns Возвращает RAW всего документа.
     */
    save(): Uint8Array;
    /**
     * Создает копию текущего документа согласно указанным правилам.
     *
     * @returns Возвращает новый документ.
     */
    copy(params?: CopyParams): Promise<CopyResponse>;
    /**
     * Создает новый XRef если предыдущий сохранен.
     * @returns
     */
    createUpdate(): void;
    /**
     * Создание и добавление Indirect Object в документ.
     * @param value Содержимое Indirect Object.
     * @param type Указывает тип Indirect Object: compressed, in-use, free.
     * @returns
     */
    createIndirectObject(value: PdfObject, type?: IndirectObjectType): PdfIndirectObject;
    /**
     * Создание пустого PDF документа с двумя объектами: Root и Pages.
     * @param xrefType Указывает тип структуры XRef.
     * @returns
     */
    static create(xrefType?: XRefType): Document;
    /**
     * Добавление шрифта в документ.
     * @param fontData
     * @param name
     * @returns
     */
    addFont(fontData: Uint8Array, name: string): Font;
    private createBfChars;
    private createCmap;
    /**
     * Поиск всех страниц документа.
     * @returns Возвращает map страниц с ключом по номеру страниц. Страницы в документе начинаются с 1.
     */
    private findPagesList;
    /**
     * Поиск всех страниц в документе. Рекурсивный метод.
     * Страницы в документе начинаются с 1.
     * @param dict
     * @returns
     */
    private findPageDict;
    /**
     * Поиск страницы по номеру.
     * @param numPage Номер страницы. Страницы в документе начинаются с 1.
     * @returns
     */
    findPage(numPage: number): PdfDictionary | null;
    /**
     * Получение всех страниц документа.
     * @returns Возвращает map страниц с ключом по номеру страниц. Страницы в документе начинаются с 1.
     */
    getPagesList(): PagesList;
    /**
     * Получение страницы по номеру.
     * @param numPage Номер страницы. Страницы в документе начинаются с 1.
     * @returns
     */
    getPage(numPage: number): PdfDictionary;
    /**
     * Создает createNull.
     * @returns
     */
    createNull(): PdfNull;
    /**
     * Создает PdfBoolean со значением v.
     * @param v PdfBoolean значение.
     * @returns
     */
    createBoolean(v: boolean): PdfBoolean;
    /**
     * Создает PdfNumber со значением v.
     * @param v PdfNumber значение.
     * @returns
     */
    createNumber(v: number): PdfNumber;
    /**
     * Создает объект для работы со флагами типа type со значением v.
     * @param type
     * @param v
     * @returns
     */
    createFlags<T extends PdfNumber>(type: PdfObjectConstructor<T>, v?: number): T;
    /**
     * Создает PdfNumber со значением v.
     * @param v PdfNumber значение.
     * @returns
     */
    createName(v: string): PdfName;
    /**
     * Создает PdfHexString со значением v.
     * @param v PdfHexString значение.
     * @returns
     */
    createHexString(v: string | Uint8Array): PdfHexString;
    /**
     * Создает PdfLiteralString со значением v.
     * @param v PdfLiteralString значение.
     * @returns
     */
    createLiteral(v: string | Uint8Array): PdfLiteralString;
    /**
     * Создает PdfArray c вложенными объектами.
     * @param object Вкладываемые объекты.
     * @returns
     */
    createArray(...objs: PdfIndirectObjectType[]): PdfArray;
    /**
     * Создает PdfStream.
     * @param params Парамеры создания PdfStream.
     * @returns
     */
    createStream(params?: IDocumentCreateStreamParams): PdfStream;
    /**
     * Создает PdfDictionary c вложенными объектами.
     * @param object Вкладываемые объекты.
     * @returns
     */
    createDictionary(object?: Record<string, PdfIndirectObjectType>): PdfDictionary;
    /**
     * Возвращает время в формате D:20230215164332Z.
     * По умолчанию возвращает текущее время.
     * @param date
     * @returns
     */
    createFormattedDateTime(date?: Date): string;
    /**
     * Создает изображение.
     * @param imageBuf
     * @returns
     */
    createImage(imageBuf: Uint8Array): Image;
    /**
      * Создает матрицу трансформации изображения.
      * @param pageInfo
      * @param imageName
      * @returns
      */
    createImageTransformationMatrix(imageParams: Pick<IImagePage, "width" | "height" | "rotation">, imageName: string): string;
}

interface PdfObjectConstructor<T extends PdfObject = PdfObject> {
    new (param?: IPdfObjectParams): T;
    create<T extends PdfObject>(this: PdfObjectConstructor<T>, xref: XRef): T;
}
interface IPdfObjectParams {
    padding?: number;
}
declare class PdfObject extends RawData {
    #private;
    padding: number;
    parent?: PdfIndirectObjectType;
    constructor(param?: IPdfObjectParams);
    get doc(): Document;
    get xref(): XRef;
    set xref(v: XRef);
    /**
     * Делает объект PDFIndirectObject. Если уже PDFIndirectObject, то оставляет как есть.
     * @param type Указатель типа IndirectObject: compressed, in-use, free.
     * @returns
     */
    ref(type?: IndirectObjectType): this;
    /**
     * Добавляет объект в последний xref.
     *
     * этот метод вызывается каждый раз при любом изменении данных объекта.
     */
    change(): void;
    protected onChange(): void;
    static create<T extends PdfObject>(this: PdfObjectConstructor<T>, xref: XRef): T;
    protected onCreate(): void;
    protected reWriteRaw(v: PdfObjectType): void;
    protected onSetValue(_v: PdfObjectType): void;
    convert<T extends PdfObject>(type: new (params?: IPdfObjectParams) => T): T;
    protected onConvert(_obj: PdfObject): void;
}

interface IPdfCommentParams extends IPdfObjectParams {
    value?: string;
}
/**
 * Комментарий. Представляет собой строку, которая начинается с символа % и
 * заканчивается переводом строки.
 */
declare class PdfComment extends PdfObject {
    #private;
    constructor(param?: IPdfCommentParams);
    get value(): string;
    set value(v: string);
}

declare class PdfAnnotFlags extends PdfFlags<typeof AnnotFlags> {
    constructor(options?: IPdfFlagsParams<typeof AnnotFlags>);
}
declare enum AnnotFlags {
    Invisible = 1,
    Hidden = 2,
    Print = 4,
    NoZoom = 8,
    NoRotate = 16,
    NoView = 32,
    ReadOnly = 64,
    Locked = 128,
    ToggleNoView = 256,
    LockedContents = 512
}
declare enum AnnotFlagsName {
    Invisible = "Invisible",
    Hidden = "Hidden",
    Print = "Print",
    NoZoom = "NoZoom",
    NoRotate = "NoRotate",
    NoView = "NoView",
    ReadOnly = "ReadOnly",
    Locked = "Locked",
    ToggleNoView = "ToggleNoView",
    LockedContents = "LockedContents"
}

interface IPdfOperatorParams {
    tag: string;
    arguments?: PdfObject[];
}
declare class PdfOperator extends RawData {
    tag: string;
    params: PdfObject[];
    constructor(param: IPdfOperatorParams);
}

declare abstract class CharTypes {
    /**
     * Numbers: [0,1,2,3,4,5,6,7,8,9].
     */
    static integer: Uint8Array;
    /**
     * Numbers: [-,+,.,0,1,2,3,4,5,6,7,8,9].
     */
    static numbers: Uint8Array;
    /**
     * Flags: [n,f]
     */
    static flags: Uint8Array;
    /**
     * White spaces
     */
    static whiteSpaces: Uint8Array;
    /**
     * End symbols for NameString.
     */
    static endNameString: Uint8Array;
}

declare abstract class CharCodes {
    /**
     * Right angle bracket ">".
     */
    static rightAngleBracket: Uint8Array;
    /**
     * Left angle bracket "<".
     */
    static leftAngleBracket: Uint8Array;
    /**
     * Right parenthesis ")".
     */
    static rightParenthesis: Uint8Array;
    /**
     * Left parenthesis "(".
     */
    static leftParenthesis: Uint8Array;
    /**
     * Right square bracket "]".
     */
    static rightSquareBracket: Uint8Array;
    /**
     * Left square bracket "[".
     */
    static leftSquareBracket: Uint8Array;
    /**
     * Forward slash "/".
     */
    static forwardSlash: Uint8Array;
    /**
     * Forward slash "\".
     */
    static slash: Uint8Array;
    /**
     * Space " ".
     */
    static space: Uint8Array;
    /**
     * Flag "n".
     */
    static n: Uint8Array;
    /**
     * Flag "f".
     */
    static f: Uint8Array;
    /**
     * Char "R".
     */
    static r: Uint8Array;
    /**
     * Char "0".
     */
    static zero: Uint8Array;
    /**
     * Char "\n".
     */
    static endLine: Uint8Array;
    /**
     * Char "\r".
     */
    static carriageReturn: Uint8Array;
    /**
     * Char "%".
     */
    static percent: Uint8Array;
}

declare abstract class CharWords {
    /**
     * Word "<<".
     */
    static startDictionary: Uint8Array;
    /**
     * Word ">>".
     */
    static endDictionary: Uint8Array;
    /**
     * Word "false".
     */
    static false: Uint8Array;
    /**
     * Word "true".
     */
    static true: Uint8Array;
    /**
     * Word "obj".
     */
    static obj: Uint8Array;
    /**
     * Word "endobj".
     */
    static endobj: Uint8Array;
    /**
     * Word "stream".
     */
    static stream: Uint8Array;
    /**
     * Word "endstream".
     */
    static endstream: Uint8Array;
    /**
     * Word "xref".
     */
    static xref: Uint8Array;
    /**
     * Word "startxref".
     */
    static startxref: Uint8Array;
    /**
     * Word "eof".
     */
    static eof: Uint8Array;
    /**
     * Word "trailer".
     */
    static trailer: Uint8Array;
    /**
     * Word "%...."
     */
    static endHeader: Uint8Array;
    /**
     * Word "\r\n".
     */
    static eol: Uint8Array;
    /**
     * Word "%PDF-".
     */
    static pdf: Uint8Array;
    /**
     * Word "null".
     */
    static null: Uint8Array;
}

declare abstract class CharOperators {
    /**
     * Operator "q".
     */
    static q: Uint8Array;
    /**
     * Operator "Q".
     */
    static Q: Uint8Array;
    /**
     * Groupe operators "q" and "Q".
     */
    static qQ: Uint8Array;
    /**
     * Operator "ID".
     */
    static ID: Uint8Array;
    /**
     * Operator "EI".
     */
    static EI: Uint8Array;
}

declare class Image extends PdfStream {
    get width(): number;
    set width(value: number);
    get height(): number;
    set height(value: number);
    onCreate(): void;
}

declare enum Names {
    AP = "AP",
    ASCII85Decode = "ASCII85Decode",
    ASCIIHexDecode = "ASCIIHexDecode",
    AcroForm = "AcroForm",
    AdbePkcs7Detached = "adbe.pkcs7.detached",
    Adobe = "Adobe",
    AdobePPKLite = "Adobe.PPKLite",
    Annot = "Annot",
    Annots = "Annots",
    BaseFont = "BaseFont",
    BBox = "BBox",
    BitsPerComponent = "BitsPerComponent",
    ByteRange = "ByteRange",
    Catalog = "Catalog",
    CIDFontType0 = "CIDFontType0",
    CIDFontType2 = "CIDFontType2",
    ColorSpace = "ColorSpace",
    Contents = "Contents",
    Count = "Count",
    DCTDecode = "DCTDecode",
    DecodeParms = "DecodeParms",
    DeviceGray = "DeviceGray",
    Encrypt = "Encrypt",
    EtsiCadesDetached = "ETSI.CAdES.detached",
    F = "F",
    FT = "FT",
    Fields = "Fields",
    Filter = "Filter",
    First = "First",
    FlateDecode = "FlateDecode",
    FontDescriptor = "FontDescriptor",
    Font = "Font",
    Form = "Form",
    FormType = "FormType",
    FontFile2 = "FontFile2",
    FontFile3 = "FontFile3",
    Height = "Height",
    ID = "ID",
    Identity = "Identity",
    IdentityH = "Identity-H",
    Image = "Image",
    Info = "Info",
    Kids = "Kids",
    LZWDecode = "LZWDecode",
    Length = "Length",
    Metadata = "Metadata",
    N = "N",
    ObjStm = "ObjStm",
    OutputIntents = "OutputIntents",
    P = "P",
    Page = "Page",
    Pages = "Pages",
    Perms = "Perms",
    Prev = "Prev",
    Rect = "Rect",
    Resources = "Resources",
    Root = "Root",
    RunLengthDecode = "RunLengthDecode",
    Sig = "Sig",
    SigFlags = "SigFlags",
    Size = "Size",
    SubFilter = "SubFilter",
    Subtype = "Subtype",
    T = "T",
    TrueType = "TrueType",
    TrustedParams = "TrustedParams",
    Type = "Type",
    Type0 = "Type0",
    Type1 = "Type1",
    V = "V",
    W = "W",
    Widget = "Widget",
    Width = "Width",
    XObject = "XObject",
    XRef = "XRef",
    Encoding = "Encoding",
    DescendantFonts = "DescendantFonts",
    ToUnicode = "ToUnicode",
    CIDToGIDMap = "CIDToGIDMap",
    CIDSystemInfo = "CIDSystemInfo",
    FontName = "FontName",
    Flags = "Flags",
    FontBBox = "FontBBox",
    ItalicAngle = "ItalicAngle",
    Ascent = "Ascent",
    StemV = "StemV",
    Descent = "Descent",
    XHeight = "XHeight",
    CapHeight = "CapHeight",
    Square = "Square",
    OCProperties = "OCProperties",
    D = "D",
    Configs = "Configs",
    Name = "Name",
    CIDSet = "CIDSet",
    FontFile = "FontFile",
    Parent = "Parent",
    Link = "Link",
    Interpolate = "Interpolate",
    ExtGState = "ExtGState",
    TR2 = "TR2",
    TR = "TR",
    A = "A",
    AA = "AA",
    DeviceRGB = "DeviceRGB",
    DeviceCMYK = "DeviceCMYK",
    DestOutputProfile = "DestOutputProfile",
    NeedAppearances = "NeedAppearances",
    AS = "AS",
    Order = "Order",
    OCGs = "OCGs",
    Names = "Names",
    Type3 = "Type3",
    Widths = "Widths",
    Normal = "Normal",
    WinAnsiEncoding = "WinAnsiEncoding",
    MacRomanEncoding = "MacRomanEncoding",
    MissingWidth = "MissingWidth",
    FirstChar = "FirstChar",
    Differences = "Differences",
    Outlines = "Outlines"
}

interface IImagePage {
    data: Uint8Array;
    width: number;
    height: number;
    rotation: number;
}
/**
 * Параметры, необходимые для создания экземпляра класса Copier.
 */
interface CopierParams {
    /**
     * Документ в который будут копироваться элементы.
     */
    doc: Document;
}
/**
 * Правила копирования элементов PDF-документа.
 */
interface Rules {
    /**
     * Правила исключения страниц. (Исключенные страницы преобразуются в изображения).
     */
    exclusion?: RulesExclusion;
    /**
     * Правила для удаления элементов.
     */
    delete?: RuleDelete[];
    /**
     * Правила для копирования элементов.
     */
    copy?: RuleCopy[];
}
/**
 * Правила исключения страниц из обычного копирования PDF-документа.
 */
interface RulesExclusion {
    rules: RuleExclusion[];
    convert(doc: Document, numPages: number[]): Promise<Map<number, IImagePage>>;
}
/**
 * Правило исключения из обычного копирования страницы PDF документа.
 * Исключенная страница копируется через преобразование в изображение.
 */
interface RuleExclusion extends RuleBase {
    /**
     * Проверяет, является ли данная страница той, которую нужно исключить.
     * @param obj - страница, которую необходимо проверить.
     * @returns Запись изменения в документе.
     */
    detect(obj: PdfDictionary): Promise<Record$1 | null>;
}
/**
 * Правило удаления элемента из PDF-документа.
 */
interface RuleDelete extends RuleBase {
    /**
     * Проверяет, является ли данный тип объекта тем, который нужно удалить.
     * @param obj - тип объекта, который необходимо проверить.
     * @returns Запись изменения в документе.
     */
    detect(obj: PdfObjectType): Promise<Record$1 | null>;
}
/**
 * Правило копирования элемента из PDF-документа.
 */
interface RuleCopy extends RuleBase {
    /**
     * Флаг, указывающий, что после применения данного правила, остальные правила игнорируются.
     */
    only: boolean;
    /**
     * Корректирует элемент при копировании.
     * @param doc - документ, в который копируются элементы.
     * @param obj - элемент, который необходимо скопировать.
     * @returns Запись изменения в документе.
     */
    correction(doc: Document, obj: PdfDictionary): Promise<Record$1 | null>;
}
/**
 * Базовый интерфейс правил.
 */
interface RuleBase {
    title: string;
    /**
     * Флаг, указывающий, что правило отключено.
     */
    disabled?: boolean;
}
type RuleConstructor<T extends RuleBase = RuleBase> = new () => T;
/**
 * Класс копирования из одного документа в другой или в новый.
 */
declare class Copier {
    doc: Document;
    private rules;
    private mapIO;
    /**
     * Logger фиксирующий изменения в документе.
     */
    logRules: Logger;
    constructor(params: CopierParams);
    get data(): Uint8Array;
    /**
     * Копирует документ из одного файла в другой или в новый.
     * @param inputFile - документ, который необходимо скопировать.
     * @param rules - правила копирования и удаления элементов.
     * @returns Скопированный документ.
     */
    copy(inputFile: Document, rules?: Rules): Promise<Document>;
    /**
     * Копирование Trailer.
     * @param obj
     */
    private copyTrailer;
    /**
     * Копирование каталога. (Root)
     * @param obj
     */
    private copyCatalog;
    /**
     * Копирование страниц.
     * @param obj
     */
    private copyPages;
    private copyPrimitiveObjectType;
    private copyObjectType;
    private copyPdfArray;
    private copyPdfStream;
    private copyPdfDictionary;
    private copyIndirectObjectType;
    /**
     * Перебирает и применяет правила исключения к страницам документа.
     * @param doc
     * @returns
     */
    private pageExclusionRules;
    /**
     * Перебирает и применяет правила удаления к obj.
     * @param obj
     * @returns
     */
    private deletionRules;
    /**
     * Перебирает и применяет правила копирования к obj.
     * @param doc
     * @param obj
     * @returns
     */
    private copyRules;
}

type WEntry = number | number[];
declare class CIDFontWidthsConverter {
    /**
     * Конвертирует W-массив в одномерный массив
     * @param data - W-массив.
     * @returns Одномерный массив.
     */
    static flatten(data: WEntry[]): number[];
    /**
     * Конвертирует одномерный массив в W-массив
     * @param data - Одномерный массив.
     * @returns W-массив.
     */
    static expand(data: number[]): WEntry[];
    /**
     * Конвертирует W-массив в `PdfArray`
     * @param doc - Документ.
     * @param data - W-массив.
     * @returns `PdfArray`.
     */
    static toPdfArray(doc: Document, data: WEntry[]): PdfArray;
}

export { ASCII85Decode, ASCIIHexDecode, AnnotFlags, AnnotFlagsName, CIDFontWidthsConverter, Catalog, CharCodes, CharOperators, CharTypes, CharWords, ColorNames, ColorSpace, Copier, CopierParams, CopyParams, CopyResponse, CustomFont, Document, Encoding, Filter, FilterCodec, FilterName, FlateDecode, Font, FontFlags, Fonts, Header, IBaseImage, IBaseInfo, ICustomFontParams, IDocumentCreateStreamParams, IFilter, IImageJpegInfo, IImagePage, IImagePngInfo, IJpegInfo, IObjectStreamParams, IPairsNumbers, IPdfArrayParams, IPdfBooleanParams, IPdfCommentParams, IPdfDictionaryParams, IPdfFlagsParams, IPdfIndirectObjectParams, IPdfNumberParams, IPdfObjectParams, IPdfOperatorParams, IPdfStringParams, IPngInfo, IResultRead, Image, ImageAnalyzer, ImageInfo, ImageTypes, IndirectObjectType, JpegInfo, LogRules, Logger, Names, ObjectStream, Pages, PagesList, Parser, PdfAnnotFlags, PdfArray, PdfBoolean, PdfBuffer, PdfComment, PdfDictionary, PdfFlags, PdfFontFlags, PdfHexString, PdfIndirectObject, PdfIndirectObjectType, PdfLiteralString, PdfName, PdfNull, PdfNumber, PdfObject, PdfObjectConstructor, PdfObjectType, PdfOperator, PdfStream, PdfString, PdfView, PngInfo, Predictor, RawData, Record$1 as Record, Registry, RegistryTypes, RuleBase, RuleConstructor, RuleCopy, RuleDelete, RuleExclusion, Rules, RulesExclusion, StringConverter, Trailer, Writer, XRef, XRefPrevParams, XRefStream, XRefStreamPrevParams, XRefTable, XRefTrailerParams, XRefType, decodeString, ensure1ByteHex, escapeSpecialSymbols, formatHex, fromBase64, fromBinary, generateUUID, generateUniqueID, hasMoreThan1ByteChars, hasUtf16BeBOM, hasUtf8BOM, hexTo2ByteHex, hexToBinary, isEqualRaw, toBase64, toBinary, toUtf16Be, unescapeSpecialSymbols };
