/*!
 * Copyright 2023 <COPYRIGHT HOLDER>
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 * 
 */
'use strict';

var tslib = require('tslib');
var pdf = require('@trusted-pdf/pdf');

class PdfSigFlags extends pdf.PdfFlags {
    constructor(options) {
        super(exports.SigFlags, options);
    }
}
exports.SigFlags = void 0;
(function (SigFlags) {
    SigFlags[SigFlags["SignaturesExist"] = 1] = "SignaturesExist";
    SigFlags[SigFlags["AppendOnly"] = 2] = "AppendOnly";
})(exports.SigFlags || (exports.SigFlags = {}));

class Signature {
    constructor(dict, name) {
        this.dict = dict;
        this.name = name;
    }
    get visibility() {
        const f = this.dict.find(pdf.Names.F, pdf.PdfNumber);
        if (f) {
            const flags = this.dict.doc.createFlags(pdf.PdfAnnotFlags, f.value).get();
            if (flags.includes(pdf.AnnotFlagsName.Hidden)
                || flags.includes(pdf.AnnotFlagsName.Invisible)
                || flags.includes(pdf.AnnotFlagsName.NoView)
                || flags.includes(pdf.AnnotFlagsName.ToggleNoView)) {
                return false;
            }
        }
        const rect = this.dict.get(pdf.Names.Rect, pdf.PdfArray);
        for (const coordinate of rect) {
            if (coordinate instanceof pdf.PdfNumber && coordinate.value != 0) {
                return true;
            }
        }
        return false;
    }
    get pageNumber() {
        const p = this.dict.get(pdf.Names.P, pdf.PdfDictionary);
        if (!p.parent || !(p.parent instanceof pdf.PdfIndirectObject)) {
            throw new Error("Невозможно получить ID страницы");
        }
        const pID = p.parent.id;
        const list = this.dict.doc.getPagesList();
        const pageNumber = [...list.keys()].find((key) => {
            const dictionary = list.get(key);
            if (!dictionary) {
                throw new Error("Неверный ключ страницы");
            }
            if (!dictionary.parent || !(dictionary.parent instanceof pdf.PdfIndirectObject)) {
                throw new Error("Страница без ID");
            }
            return dictionary.parent.id === pID;
        });
        if (!pageNumber) {
            throw new Error("Страница не найдена");
        }
        return pageNumber;
    }
    set trustedParams(value) {
        if (value) {
            this.dict.set(pdf.Names.TrustedParams, this.dict.doc.createLiteral(value));
        }
    }
    get trustedParams() {
        const params = this.dict.find(pdf.Names.TrustedParams, pdf.PdfLiteralString);
        if (params) {
            return params.value;
        }
        return params;
    }
    async sign(options) {
        var _a;
        const doc = this.dict.doc;
        doc.save();
        if (this.dict.find(pdf.Names.V)) {
            throw new Error(`Sign(): Сигнатура '${this.name}' уже подписана`);
        }
        const catalog = doc.xref.trailer.Root;
        const acroForm = catalog.findOrSet(pdf.Names.AcroForm, pdf.PdfDictionary, true);
        let flags = acroForm.find(pdf.Names.SigFlags, PdfSigFlags);
        if (!flags) {
            flags = doc.createFlags(PdfSigFlags);
            acroForm.set(pdf.Names.SigFlags, flags);
        }
        flags.set("SignaturesExist");
        flags.set("AppendOnly");
        const count = String(doc.buffer.length).length;
        const padding = count > 10 ? count + 2 : 10;
        const indexByteRange1 = doc.createNumber(0);
        const indexByteRange2 = doc.createNumber(0);
        indexByteRange2.padding = padding;
        const indexByteRange3 = doc.createNumber(0);
        indexByteRange3.padding = padding;
        const indexByteRange4 = doc.createNumber(0);
        indexByteRange4.padding = padding;
        const contents = doc.createHexString(new Uint8Array(options.contentLength));
        this.dict.set(pdf.Names.V, doc.createDictionary({
            Type: doc.createName(pdf.Names.Sig),
            Filter: doc.createName(pdf.Names.AdobePPKLite),
            SubFilter: doc.createName(pdf.Names.EtsiCadesDetached),
            Contents: contents,
            ByteRange: doc.createArray(indexByteRange1, indexByteRange2, indexByteRange3, indexByteRange4),
            M: doc.createLiteral(doc.createFormattedDateTime()),
        }).ref());
        await ((_a = options.onUpdate) === null || _a === void 0 ? void 0 : _a.call(options));
        doc.save();
        indexByteRange2.reWriteValue(contents.raw.offset);
        const indexOfEndContent = contents.raw.offset + contents.raw.length;
        indexByteRange3.reWriteValue(indexOfEndContent);
        indexByteRange4.reWriteValue(doc.buffer.length - indexOfEndContent);
        const raw1 = doc.buffer.subarray(indexByteRange1.value, indexByteRange1.value + indexByteRange2.value).getRaw();
        const raw2 = doc.buffer.subarray(indexByteRange3.value, indexByteRange3.value + indexByteRange4.value).getRaw();
        const rawContent = new Uint8Array(raw1.length + raw2.length);
        rawContent.set(raw1);
        rawContent.set(raw2, raw1.length);
        const cms = await options.onSign(rawContent);
        const newCms = new Uint8Array((contents.raw.length - 2) / 2);
        newCms.set(cms, 0);
        contents.reWriteValue(newCms);
    }
    async verify(options) {
        const res = {
            verifyCallBack: false,
        };
        const doc = this.dict.doc;
        const v = this.dict.get(pdf.Names.V, pdf.PdfDictionary);
        const byteRange = v.get(pdf.Names.ByteRange, pdf.PdfArray);
        const indexByteRange1 = byteRange.get(0, pdf.PdfNumber).value;
        const indexByteRange2 = byteRange.get(1, pdf.PdfNumber).value;
        const indexByteRange3 = byteRange.get(2, pdf.PdfNumber).value;
        const indexByteRange4 = byteRange.get(3, pdf.PdfNumber).value;
        const raw1 = doc.buffer.subarray(indexByteRange1, indexByteRange1 + indexByteRange2).getRaw();
        const raw2 = doc.buffer.subarray(indexByteRange3, indexByteRange3 + indexByteRange4).getRaw();
        const rawContent = new Uint8Array(raw1.length + raw2.length);
        rawContent.set(raw1);
        rawContent.set(raw2, raw1.length);
        const sign = v.get(pdf.Names.Contents, pdf.PdfHexString).toBinary();
        res.verifyCallBack = await options.onVerify(rawContent, sign);
        return res;
    }
}

var _Sign_signatures;
class Sign {
    constructor(doc) {
        this.doc = doc;
        _Sign_signatures.set(this, new Map());
        this.findSignaturesInDocument();
    }
    verifyAttestation() {
        let status = true;
        const log = [];
        const listXref = [this.doc.xref];
        let current = this.doc.xref;
        while (true) {
            if (!current.prev) {
                break;
            }
            listXref.push(current.prev);
            current = current.prev;
        }
        listXref.reverse();
        const listPagesAndContents = [];
        const listWidgets = new Map();
        let find = false;
        for (const xref of listXref) {
            if (find) {
                for (const [key, obj] of xref.objects) {
                    if (listPagesAndContents.includes(key)) {
                        status = false;
                        log.push(`Объект ${key} был изменен`);
                    }
                    const oldRect = listWidgets.get(key);
                    if (oldRect) {
                        if (!(obj.value instanceof pdf.PdfDictionary)) {
                            log.push(`Виджет ${key} перестал быть Dictionary`);
                            status = false;
                            return { status, log };
                        }
                        const subtype = obj.value.find(pdf.Names.Subtype, pdf.PdfName);
                        if (!subtype || subtype.value !== pdf.Names.Widget) {
                            log.push(`Виджет ${key} перестал быть Widget`);
                            status = false;
                            return { status, log };
                        }
                        const newRect = this.getRect(obj.value);
                        if (oldRect.toString() !== newRect.toString()) {
                            log.push(`Виджет ${key} изменил rect ${oldRect} на rect ${newRect}`);
                            status = false;
                            return { status, log };
                        }
                    }
                }
            }
            if (!find) {
                for (const [key, obj] of xref.objects) {
                    const value = obj.getValue(xref);
                    if (value instanceof pdf.PdfDictionary) {
                        const type = value.find(pdf.Names.Type, pdf.PdfName);
                        if (type) {
                            if (type.value === pdf.Names.Pages) {
                                listPagesAndContents.push(key);
                                continue;
                            }
                            if (type.value === pdf.Names.Page) {
                                listPagesAndContents.push(key);
                                this.findWidgetsRectOnPage(xref, value, listWidgets);
                                const contents = value.find(pdf.Names.Contents);
                                if (contents && contents.parent && contents.parent instanceof pdf.PdfIndirectObject) {
                                    listPagesAndContents.push(contents.parent.id);
                                }
                                continue;
                            }
                            if (type.value.toLowerCase() === "sig") {
                                find = true;
                            }
                        }
                    }
                }
            }
        }
        if (!find) {
            status = false;
            log.push("Подписи не найдены");
        }
        return { status, log };
    }
    findWidgetsRectOnPage(xref, page, map) {
        const annots = page.find(pdf.Names.Annots, pdf.PdfArray);
        if (!annots) {
            return null;
        }
        if (!map) {
            map = new Map();
        }
        for (let index = 0; index < annots.length; index++) {
            const annot = annots.get(index, pdf.PdfDictionary);
            const subtype = annot.find(pdf.Names.Subtype, pdf.PdfName);
            if (!subtype || subtype.value !== pdf.Names.Widget) {
                break;
            }
            if (!annot.parent || !(annot.parent instanceof pdf.PdfIndirectObject)) {
                break;
            }
            const id = annot.parent.id;
            const history = xref.objects.get(id);
            if (!history || !(history.value instanceof pdf.PdfDictionary)) {
                throw new Error(`Невозможно получить аннотацию ${id} из обновления`);
            }
            const arr = this.getRect(history.value);
            map.set(id, arr);
        }
        if (!map.size) {
            return null;
        }
        return map;
    }
    getRect(obj) {
        const arr = new Array();
        const rect = obj.find(pdf.Names.Rect, pdf.PdfArray);
        if (!rect) {
            return arr;
        }
        for (let index = 0; index < rect.length; index++) {
            const element = rect.get(index, pdf.PdfNumber).value;
            arr.push(element);
        }
        return arr;
    }
    findSignaturesInDocument() {
        const catalog = this.doc.xref.trailer.Root;
        const acroForm = catalog.find(pdf.Names.AcroForm, pdf.PdfDictionary);
        if (!acroForm) {
            return;
        }
        const fields = acroForm.get(pdf.Names.Fields, pdf.PdfArray);
        for (let index = 0; index < fields.length; index++) {
            const dict = fields.get(index, pdf.PdfDictionary);
            const ft = dict.find(pdf.Names.FT, pdf.PdfName);
            if (!ft || ft.value !== pdf.Names.Sig) {
                continue;
            }
            const name = dict.get(pdf.Names.T, pdf.PdfString).value;
            tslib.__classPrivateFieldGet(this, _Sign_signatures, "f").set(name, new Signature(dict, name));
        }
    }
    getSignaturesList() {
        return tslib.__classPrivateFieldGet(this, _Sign_signatures, "f");
    }
    findSignature(key) {
        var _a, _b;
        if (typeof key === "string") {
            return (_a = tslib.__classPrivateFieldGet(this, _Sign_signatures, "f").get(key)) !== null && _a !== void 0 ? _a : null;
        }
        const keys = Array.from(tslib.__classPrivateFieldGet(this, _Sign_signatures, "f").keys());
        return (_b = tslib.__classPrivateFieldGet(this, _Sign_signatures, "f").get(keys[key])) !== null && _b !== void 0 ? _b : null;
    }
    getSignature(key) {
        const sig = this.findSignature(key);
        if (!sig) {
            throw new Error(`${key} signature не найдена`);
        }
        return sig;
    }
    addSignatures(...params) {
        if (!params.length) {
            throw new Error("addSignature(): должен содержать параметры");
        }
        const pages = this.doc.getPagesList();
        const list = new Map();
        for (const param of params) {
            const page = pages.get(param.page);
            if (!page) {
                throw new Error(`Страница ${param.page} не найдена`);
            }
            if (tslib.__classPrivateFieldGet(this, _Sign_signatures, "f").has(param.name)) {
                throw new Error(`Сигнатура с имененем ${param.name}, не найдена`);
            }
            const sigDict = this.addSignatureOnPage(page, param.name, Object.assign({}, Sign.DEFAULT_RECT, param.rectangle), param.stream);
            const sig = new Signature(sigDict, param.name);
            if (param.trustedParams) {
                sig.trustedParams = param.trustedParams;
            }
            tslib.__classPrivateFieldGet(this, _Sign_signatures, "f").set(param.name, sig);
            list.set(param.name, sig);
        }
        return list;
    }
    addSignatureOnPage(page, name, rectangle, stream = new Uint8Array()) {
        const catalog = this.doc.xref.trailer.Root;
        let sigName = name.trim();
        if (sigName === "") {
            sigName = "xxxxxxxx-xxxx-4xxx-yxxx-xxxxxxxxxxxx".replace(/[xy]/g, (c) => {
                const r = Math.random() * 16 | 0;
                const v = c === "x" ? r : (r & 0x3 | 0x8);
                return v.toString(16);
            });
        }
        const sig = this.doc.createDictionary({
            FT: this.doc.createName(pdf.Names.Sig),
            T: this.doc.createLiteral(sigName),
            F: this.doc.createNumber(4),
            Type: this.doc.createName(pdf.Names.Annot),
            Subtype: this.doc.createName(pdf.Names.Widget),
            P: page.ref(),
            Rect: this.doc.createArray(this.doc.createNumber(rectangle.x), this.doc.createNumber(rectangle.y), this.doc.createNumber(rectangle.x + rectangle.width), this.doc.createNumber(rectangle.y + rectangle.height)),
        });
        if (rectangle.height || rectangle.width) {
            sig.set(pdf.Names.AP, this.doc.createDictionary({
                N: this.doc.createStream({
                    dict: {
                        Type: this.doc.createName(pdf.Names.XObject),
                        Subtype: this.doc.createName(pdf.Names.Form),
                        BBox: this.doc.createArray(this.doc.createNumber(0), this.doc.createNumber(0), this.doc.createNumber(rectangle.width), this.doc.createNumber(rectangle.height)),
                        FormType: this.doc.createNumber(1),
                        Resources: this.doc.createDictionary(),
                    },
                    stream: stream,
                }).ref(),
            }));
        }
        page
            .findOrSet(pdf.Names.Annots, pdf.PdfArray)
            .push(sig.ref());
        const acroForm = catalog.findOrSet(pdf.Names.AcroForm, pdf.PdfDictionary, true);
        acroForm.findOrSet(pdf.Names.Fields, pdf.PdfArray).push(sig.ref());
        let sigFlags = acroForm.find(pdf.Names.SigFlags, PdfSigFlags);
        if (!sigFlags) {
            sigFlags = this.doc.createFlags(PdfSigFlags);
            acroForm.set(pdf.Names.SigFlags, sigFlags);
        }
        sigFlags.set("SignaturesExist");
        return sig;
    }
}
_Sign_signatures = new WeakMap();
Sign.DEFAULT_RECT = { x: 0, y: 0, height: 0, width: 0 };

exports.PdfSigFlags = PdfSigFlags;
exports.Sign = Sign;
exports.Signature = Signature;
